#!/usr/bin/env python
# coding=utf-8
# Copyright 2016 Flowdas Inc. <prospero@flowdas.com>
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import sys

import click
import gunicorn.app.base
from gunicorn import util


class Application(gunicorn.app.base.BaseApplication):
    def __init__(self, app, options):
        self.options = options
        self.app_uri = app
        super(Application, self).__init__()

    def load_config(self):
        for key, value in self.options.items():
            if key in self.cfg.settings and value is not None:
                self.cfg.set(key.lower(), value)

    def load(self):
        return util.import_app(self.app_uri)


@click.command()
# @click.option('--log-syslog-facility', help='Syslog facility name. [user]')
# @click.option('--access-logfile', help='The Access log file to write to. [None]')
# @click.option('--timeout', default=30,
#               help='Workers silent for more than this many seconds are killed and restarted. [30]')
@click.option('--workers', default=1, help='The number of worker processes for handling requests. [1]')
# @click.option('--log-level', default='info', help='The granularity of Error log outputs. [info]')
# @click.option('--capture-output', is_flag=True, help='Redirect stdout/stderr to Error log. [False]')
@click.option('--name', default='oliver', help='A base to use with setproctitle for process naming. [oliver]')
# @click.option('--access-logformat',
#               help='The access log format. [%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"]')
# @click.option('--log-syslog-to', help='Address to send syslog messages. [unix:///var/run/syslog]')
# @click.option('--log-syslog', is_flag=True, help='Send logs to syslog. [False]')
# @click.option('--statsd-host', help='``host:port`` of the statsd server to log to. [None]')
# @click.option('--pid', help='A filename to use for the PID file. [None]')
# @click.option('--umask', help='A bit mask for the file mode on files written by Oliver. [0]')
# @click.option('--log-config', help='The log config file to use. [None]')
@click.option('--bind', help='The socket to bind. [["127.0.0.1:8000"]]')
# @click.option('--max-requests', default=0,
#               help='The maximum number of requests a worker will process before restarting. [0]')
# @click.option('--env', help='Set environment variable (key=value). [[]]')
@click.option('--pythonpath', help='A comma-separated list of directories to add to the Python path. [None]')
# @click.option('--group', help='Switch worker process to run as this group.')
@click.option('--daemon', is_flag=True, help='Daemonize the Gunicorn process. [False]')
# @click.option('--log-syslog-prefix',
#               help='Makes Oliver use the parameter as program-name in the syslog entries. [None]')
# @click.option('--user', help='Switch worker processes to run as this user.')
# @click.option('--graceful-timeout', default=30, help='Timeout for graceful workers restart. [30]')
@click.option('--backlog', default=2048, help='The maximum number of pending connections. [2048]')
# @click.option('--config', help='The Gunicorn config file. [None]')
@click.option('--worker-connections', default=10000, help='The maximum number of simultaneous clients. [10000]')
# @click.option('--chdir', help='Chdir to specified directory before apps loading.')
# @click.option('--error-logfile', help='The Error log file to write to. [-]')
# @click.option('--statsd-prefix', help='Prefix to use when emitting statsd metrics (a trailing ``.`` is added, []')
@click.argument('app')
def main(app, pythonpath=None, **options):
    options.update(worker_class='flowdas.oliver.gunicorn')
    if pythonpath:
        sys.path.extend(pythonpath.split(','))
    Application(app, options).run()


if __name__ == '__main__':
    main()
